/***************************************************************************
                          kde_interface.cpp  -  description
                             -------------------
    begin                : Sun Mar 25 2001
    copyright            : (C) 2001 by andres
    email                : dae@chez.com
 ***************************************************************************/

#include "kde_disc.h"
#include "kde_interface.h"
#include "kde_net.h"
#include "kde_menu.h"
#include "kde_slider.h"
#include "kde_preferences.h"
#include "intf_eject.h"

#include "video.h"
#include "video_output.h"

#include <iostream.h>

#include <kaction.h>
#include <kfiledialog.h>
#include <klocale.h>
#include <kmessagebox.h>
#include <kstdaction.h>
#include <kurl.h>
#include <kurldrag.h>
#include <qcursor.h>
#include <qdragobject.h>
#include <qstringlist.h>
#include <qtimer.h>
#include <kdialog.h>

#define ID_STATUS_MSG       1
#define ID_DATE             2
#define ID_STREAM_SOURCE    3



KInterface::KInterface( intf_thread_t *p_intf, QWidget *parent,
        const char *name ) : KMainWindow(parent,name)
{
    setAcceptDrops(true);

    this->p_intf = p_intf;

    fDiskDialog = new KDiskDialog( this );
    fNetDialog = new KNetDialog( this );
    fTitleMenu = new KTitleMenu( p_intf, this );

    fSlider = new KVLCSlider( QSlider::Horizontal, this );
    fSlider->setMaxValue(10000);
    connect( fSlider, SIGNAL( userChanged( int ) ),
             this, SLOT( slotSliderMoved( int ) ) );
    connect( fSlider, SIGNAL( valueChanged( int ) ),
             this, SLOT( slotSliderChanged( int ) ) );
    setCentralWidget(fSlider);

    fTimer = new QTimer( this );
    connect( fTimer, SIGNAL( timeout() ), this, SLOT( slotManage() ) );
    fTimer->start( 100 );

    resize( 400, 30 );

    ///////////////////////////////////////////////////////////////////
    // call inits to invoke all other construction parts
    // XXX could we move this up ?
    initStatusBar();
    initActions();

    dvdToolBar = toolBar( "dvdToolBar" );


    // add certain calls to the popup menu
    fileOpen->plug( fTitleMenu );
    fileOpenRecent->plug( fTitleMenu );
    diskOpen->plug( fTitleMenu );
    streamOpen->plug( fTitleMenu );
    fullscreen->plug( fTitleMenu );
    play->plug( fTitleMenu );
    pause->plug( fTitleMenu );
    slow->plug( fTitleMenu );
    fast->plug( fTitleMenu );
    fileClose->plug( fTitleMenu );
    fileQuit->plug( fTitleMenu );
}

KInterface::~KInterface()
{
    ;
}

void KInterface::initActions()
{
    fileOpen = KStdAction::open( this, SLOT(slotFileOpen()),
                                 actionCollection());
    fileOpenRecent =
        KStdAction::openRecent(this, SLOT(slotFileOpenRecent(const KURL&)),
                               actionCollection());
    fileClose = KStdAction::close(this, SLOT(slotFileClose()),
                                  actionCollection());
    preferences = KStdAction::preferences(this, SLOT(slotShowPreferences()),
                                          actionCollection());
    fileQuit = KStdAction::quit(this, SLOT(slotFileQuit()), actionCollection());
    viewToolBar = KStdAction::showToolbar(this, SLOT(slotViewToolBar()),
                                          actionCollection() );
    viewStatusBar = KStdAction::showStatusbar( this, SLOT(slotViewStatusBar()),
                                              actionCollection() );

    ejectDisc = new KAction( _( "Eject Disc"), 0, 0, this,
                             SLOT( slotEjectDisc()), actionCollection(),
                             "eject_disc" );
    diskOpen = new KAction( _( "Open &Disk" ), 0, 0, this,
                            SLOT( slotOpenDisk() ), actionCollection(),
                            "open_disk" );
    streamOpen = new KAction( _( "Open &Stream" ), 0, 0, this,
                              SLOT( slotOpenStream() ), actionCollection(),
                              "open_stream" );
    backward = new KAction( _( "&Backward" ), 0, 0, this,
                            SLOT( slotNull() ), actionCollection(),
                            "backward" );
    backward->setEnabled( false );
    stop = new KAction( _( "&Stop" ), 0, 0, this, SLOT( slotStop() ),
                        actionCollection(), "stop" );
    play = new KAction( _( "&Play" ), 0, 0, this, SLOT( slotPlay() ),
                        actionCollection(), "play" );
    pause = new KAction( _( "P&ause" ), 0, 0, this, SLOT( slotPause() ),
                         actionCollection(), "pause" );
    slow = new KAction( _( "&Slow" ), 0, 0, this, SLOT( slotSlow() ),
                        actionCollection(), "slow" );
    fast = new KAction( _( "Fas&t" ), 0, 0, this, SLOT( slotFast() ),
                        actionCollection(), "fast" );
    prev = new KAction( _( "Prev" ), 0, 0, this, SLOT( slotPrev() ),
                        actionCollection(), "prev" );
    next = new KAction( _( "Next" ), 0, 0, this, SLOT( slotNext() ),
                        actionCollection(), "next" );
    nextTitle = new KAction( _( "Next" ), 0, 0, this, SLOT( slotTitleNext() ),
                             actionCollection(), "title_next" );
    prevTitle = new KAction( _( "Prev" ), 0, 0, this, SLOT( slotTitlePrev() ),
                             actionCollection(), "title_prev" );
    nextChapter = new KAction( _( "Up" ), 0, 0, this,
                               SLOT( slotChapterNext() ),
                               actionCollection(), "chapter_next" );
    prevChapter = new KAction( _( "Down" ), 0, 0, this,
                               SLOT( slotTitleNext() ),
                               actionCollection(), "chapter_prev" );

    fullscreen = new KAction( _( "Fullscreen" ), 0, 0, this,
                              SLOT( slotFullscreen() ),
                              actionCollection(), "view_fullscreen" );
    hideInterface = new KAction( _( "Hide Interface" ), 0, 0, this,
                                 SLOT( slotNull() ),
                                 actionCollection(), "view_hide_interface" );
    hideInterface->setEnabled( false );
    modules = new KAction( _( "Modules..."), 0, 0, this,
                           SLOT( slotNull() ),
                           actionCollection(), "view_modules" );
    modules->setEnabled(false);
    messages = new KAction( _( "Messages..."), 0, 0, this,
                            SLOT( slotNull() ),
                            actionCollection(), "view_messages" );
    messages->setEnabled(false);
    showPlaylist = new KAction( _( "Playlist..."), 0, 0, this,
                                SLOT( slotNull() ),
                                actionCollection(), "view_playlist" );
    showPlaylist->setEnabled(false);

    viewChapter = new KSelectAction( _( "Chapter" ), 0, this,
                                     SLOT( slotNull() ), actionCollection(),
                                     "view_chapter" );
    viewProgram = new KSelectAction( _( "Program" ), 0, this,
                                     SLOT( slotNull() ), actionCollection(),
                                     "view_program" );
    viewTitle = new KSelectAction( _( "Title" ), 0, this,
                                     SLOT( slotNull() ), actionCollection(),
                                     "view_title" );
    audio = new KSelectAction( _( "Audio" ), 0, this, SLOT( slotNull() ),
                               actionCollection(), "audio" );
    subtitles = new KSelectAction( _( "Subtitles" ), 0, this,
                                  SLOT( slotNull() ),
                                  actionCollection(), "subtitles" );
    
    ejectDisc->setStatusText( _("Eject Disc"));
    fileOpen->setStatusText(_("Open a File"));
    fileOpenRecent->setStatusText(_("Opens a recently used file"));
    fileClose->setStatusText(_("Closes the actual document"));
    fileQuit->setStatusText(_("Quits the application"));
    viewToolBar->setStatusText(_("Enables/disables the toolbar"));
    viewStatusBar->setStatusText(_("Enables/disables the statusbar"));

    diskOpen->setStatusText( _( "Open a DVD or VCD") );
    streamOpen->setStatusText( _( "Select a Network Stream" ) );
    backward->setStatusText( _( "Backward" ) );
    stop->setStatusText( _( "Stop Stream" ) );
    play->setStatusText( _( "Play Stream" ) );
    pause->setStatusText( _( "Pause Stream" ) );
    slow->setStatusText( _( "Slow" ) );
    fast->setStatusText( _( "Fast" ) );
    prev->setStatusText( _( "Prev" ) );
    next->setStatusText( _( "Next" ) );

    createGUI(DATA_PATH "/kde_ui.rc");
}

void KInterface::initStatusBar()
{
  ///////////////////////////////////////////////////////////////////
  // STATUSBAR
  // TODO: add your own items you need for displaying current application status.
    statusBar()->insertItem(_("Ready."), ID_STATUS_MSG, 1, false);
    statusBar()->setItemAlignment( ID_STATUS_MSG, AlignLeft | AlignVCenter );
    statusBar()->insertItem( "0:00:00", ID_DATE, 0, true );
}

/////////////////////////////////////////////////////////////////////
// SLOT IMPLEMENTATION
/////////////////////////////////////////////////////////////////////

/* "internal" slots */
void KInterface::slotStatusMsg(const QString &text)
{
    ///////////////////////////////////////////////////////////////////
    // change status message permanently
    statusBar()->clear();
    statusBar()->changeItem(text, ID_STATUS_MSG);
}

void KInterface::slotManage()
{
    vlc_mutex_lock( &p_intf->change_lock );

    /* If the "display popup" flag has changed */
    if( p_intf->b_menu_change )
    {
        fTitleMenu->popup( ( QCursor::pos() ) );
        p_intf->b_menu_change = 0;
    }

    if( p_input_bank->pp_input[0] != NULL )
    {
        vlc_mutex_lock( &p_input_bank->pp_input[0]->stream.stream_lock );
        if( p_input_bank->pp_input[0]->stream.b_changed )
        {
            intf_WarnMsg(1,"modemanage");
            ModeManage();
            intf_WarnMsg(1,"updatemenus");
            slotUpdateMenus();
            p_input_bank->pp_input[0]->stream.b_changed = 0;
        }

        /* Manage the slider */
#define p_area p_input_bank->pp_input[0]->stream.p_selected_area
        if( p_area->i_size != 0 )
        {
            intf_WarnMsg(1,"setValue");
            fSlider->setValue( ( 10000. * p_area->i_tell ) / p_area->i_size );
        }
#undef p_area
        vlc_mutex_unlock( &p_input_bank->pp_input[0]->stream.stream_lock );
                
    }

    /* Manage core vlc functions through the callback */
    p_intf->pf_manage(p_intf);
    
    if( p_intf->b_die )
    {
        p_intf->p_sys->p_app->quit();
    }
    
    vlc_mutex_unlock( &p_intf->change_lock );
}

void KInterface::slotUpdateMenus()
{
        /* look for selected ES */
    es_descriptor_t   * p_audio_es;
    es_descriptor_t   * p_spu_es;
    int i;
    p_audio_es = NULL;
    p_spu_es = NULL;

    p_intf->p_sys->b_chapter_update |= p_intf->p_sys->b_title_update;
    p_intf->p_sys->b_audio_update |= p_intf->p_sys->b_title_update |
                                     p_intf->p_sys->b_program_update;
    p_intf->p_sys->b_spu_update |= p_intf->p_sys->b_title_update |
                                   p_intf->p_sys->b_program_update;

    intf_WarnMsg(1, "1");
    for( i = 0; i < p_input_bank->pp_input[0]->stream.i_selected_es_number; i++ )
    {
        intf_WarnMsg(1, "1.1 %d", i);
        if ( p_input_bank == NULL) { intf_WarnMsg(1, "f1");}
        if ( p_input_bank->pp_input[0] == NULL) { intf_WarnMsg(1, "f2");}
        if ( p_input_bank->pp_input[0]->stream.pp_selected_es[i] == NULL) { intf_WarnMsg(1, "f3");}
        if( p_input_bank->pp_input[0]->stream.pp_selected_es[i]->i_cat
            == AUDIO_ES )
        {
            intf_WarnMsg(1, "1.1.1");

            p_audio_es = p_input_bank->pp_input[0]->stream.pp_selected_es[i];
//            p_intfGlobal->p_sys->p_audio_es_old = p_audio_es;
        
        }
        intf_WarnMsg(1, "1.2");

        if( p_input_bank->pp_input[0]->stream.pp_selected_es[i]->i_cat
            == SPU_ES )
        {
            p_spu_es = p_input_bank->pp_input[0]->stream.pp_selected_es[i];
            //          p_intfGlobal->p_sys->p_spu_es_old = p_spu_es;
        }
    }
    intf_WarnMsg(1, "2");

//    vlc_mutex_unlock( &p_input_bank->pp_input[0]->stream.stream_lock );

    /* audio menus */
    if( p_intf->p_sys->b_audio_update )
    {
        LanguageMenu( audio, p_audio_es, AUDIO_ES );
        p_intf->p_sys->b_audio_update = false;
    }

    /* sub picture menus */
    if( p_intf->p_sys->b_spu_update )
    {
        LanguageMenu( subtitles, p_spu_es, SPU_ES );
        p_intf->p_sys->b_spu_update = false;
    }

    //  vlc_mutex_lock( &p_input_bank->pp_input[0]->stream.stream_lock );
    p_intf->p_sys->b_title_update = false;
    intf_WarnMsg(1, "3");

}

void KInterface::slotSliderChanged( int position )
{
    if( p_input_bank->pp_input[0] != NULL )
    {
        char psz_time[ OFFSETTOTIME_MAX_SIZE ];


#define p_area p_input_bank->pp_input[0]->stream.p_selected_area
        statusBar()->changeItem( input_OffsetToTime( p_input_bank->pp_input[0],
                                                     psz_time,
                                                     ( p_area->i_size *
                                                       position ) / 10000 ),
                                 ID_DATE );
#undef p_area

     }
}


/* user actions */

void KInterface::slotSliderMoved( int position )
{
// XXX is this locking really useful ?
    vlc_mutex_lock( &p_intf->change_lock );

    off_t i_seek = ( position * p_input_bank->pp_input[0]->stream.p_selected_area->i_size ) / 10000;
    input_Seek( p_input_bank->pp_input[0], i_seek );

    vlc_mutex_unlock( &p_intf->change_lock );
}

void KInterface::slotFileOpen()
{
    slotStatusMsg( _( "Opening file..." ) );
    KURL url=KFileDialog::getOpenURL( config_GetPszVariable( "search-path" ),
            _( "*|All files" ), this, _( "Open File..." ) );

    if( !url.isEmpty() )
    {
        fileOpenRecent->addURL( url );
        intf_PlaylistAdd( p_main->p_playlist, PLAYLIST_END,
                          url.path().latin1() );
        intf_PlaylistJumpto( p_main->p_playlist, p_main->p_playlist->i_size-2 );
    }

    slotStatusMsg( _( "Ready." ) );
}

void KInterface::slotFileOpenRecent(const KURL& url)
{
    slotStatusMsg( _( "Opening file..." ) );
    slotStatusMsg( _( "Ready." ) );
}

void KInterface::slotFileClose()
{
    slotStatusMsg( _( "Closing file..." ) );
    
    close();

    slotStatusMsg( _( "Ready." ) );
}

void KInterface::slotFileQuit()
{
    slotStatusMsg( _( "Exiting..." ) );
    p_intf->p_sys->p_app->quit();
    slotStatusMsg( _( "Ready." ) );
}

void KInterface::slotViewToolBar()
{
    slotStatusMsg(_("Toggling toolbar..."));
    ///////////////////////////////////////////////////////////////////
    // turn Toolbar on or off
    if(!viewToolBar->isChecked())
    {
        toolBar("mainToolBar")->hide();
    }
    else
    {
        toolBar("mainToolBar")->show();
    }        

    slotStatusMsg(_("Ready."));
}

void KInterface::slotViewStatusBar()
{
    slotStatusMsg(_("Toggle the statusbar..."));
    ///////////////////////////////////////////////////////////////////
    //turn Statusbar on or off
    if(!viewStatusBar->isChecked())
    {
        statusBar()->hide();
    }
    else
    {
        statusBar()->show();
    }

    slotStatusMsg(_("Ready."));
}

void KInterface::slotShowPreferences()
{
    // Do something
    KPreferences("main", this, "preferences");
}

void KInterface::slotOpenDisk()
{
    int r = fDiskDialog->exec();
    if ( r )
    {
        // Build source name
        QString source;
        source += fDiskDialog->type();
        source += ':';
        source += fDiskDialog->device();

        source += '@';
        source += fDiskDialog->title();
        source += ',';
        source += fDiskDialog->chapter();

        // add it to playlist
        intf_PlaylistAdd( p_main->p_playlist, PLAYLIST_END, source.latin1() );

        // Select added item and switch to disk interface
        intf_PlaylistJumpto( p_main->p_playlist, p_main->p_playlist->i_size-2 );
        if( p_input_bank->pp_input[0] != NULL )
        {
            p_input_bank->pp_input[0]->b_eof = 1;
        }
    }
}

void KInterface::slotOpenStream()
{
    int r = fNetDialog->exec();
    if ( r )
    {
        // Build source name
        QString source;
        source += fNetDialog->protocol();
        source += "://";
        source += fNetDialog->server();
        source += ":";
        source += QString().setNum( fNetDialog->port() );

        // add it to playlist
        intf_PlaylistAdd( p_main->p_playlist, PLAYLIST_END, source.latin1() );
        intf_PlaylistJumpto( p_main->p_playlist, p_main->p_playlist->i_size-2 );

        if( p_input_bank->pp_input[0] != NULL )
        {
            p_input_bank->pp_input[0]->b_eof = 1;
        }
    }
}

void KInterface::slotNull()
{
    intf_ErrMsg( "kde interface: Unimplemented action!" );
}


void KInterface::dragEnterEvent( QDragEnterEvent *event )
{
    event->accept( QUriDrag::canDecode( event ) );
}

void KInterface::dropEvent( QDropEvent *event )
{
    KURL::List urlList;

    if ( KURLDrag::decode( event, urlList ) )
    {
        for ( KURL::List::ConstIterator i = urlList.begin();
              i != urlList.end(); i++ )
        {
            // XXX add a private function to add a KURL with checking
            // actually a whole class for core abstraction would be neat
            if( !(*i).isEmpty() )
            {
                fileOpenRecent->addURL( *i );
                intf_PlaylistAdd( p_main->p_playlist, PLAYLIST_END,
                                  (*i).path() );
            }
        }
    }
}

/* the rest of the code here is "copied" from the gtk module */ 
void KInterface::slotPlay()
{
    if( p_input_bank->pp_input[0] != NULL )
    {
        input_SetStatus( p_input_bank->pp_input[0], INPUT_STATUS_PLAY );
        p_main->p_playlist->b_stopped = 0;
    }
    else
    {
        vlc_mutex_lock( &p_main->p_playlist->change_lock );

        if( p_main->p_playlist->b_stopped )
        {
            if( p_main->p_playlist->i_size )
            {
                vlc_mutex_unlock( &p_main->p_playlist->change_lock );
                intf_PlaylistJumpto( p_main->p_playlist,
                                     p_main->p_playlist->i_index );
            }
            else
            {
                vlc_mutex_unlock( &p_main->p_playlist->change_lock );
                slotFileOpen();
            }
        }
        else
        {

            vlc_mutex_unlock( &p_main->p_playlist->change_lock );
        }

    }

    return;
}

void KInterface::slotPause()
{
    if( p_input_bank->pp_input[0] != NULL )
    {
        input_SetStatus( p_input_bank->pp_input[0], INPUT_STATUS_PAUSE );

        vlc_mutex_lock( &p_main->p_playlist->change_lock );
        p_main->p_playlist->b_stopped = 0;
        vlc_mutex_unlock( &p_main->p_playlist->change_lock );
    }
}

void KInterface::slotStop()
{
    if( p_input_bank->pp_input[0] != NULL )
    {
        /* end playing item */
        p_input_bank->pp_input[0]->b_eof = 1;

        /* update playlist */
        vlc_mutex_lock( &p_main->p_playlist->change_lock );

        p_main->p_playlist->i_index--;
        p_main->p_playlist->b_stopped = 1;

        vlc_mutex_unlock( &p_main->p_playlist->change_lock );
    }
}

void KInterface::slotPrev()
{
    if( p_input_bank->pp_input[0] != NULL )
    {
        /* FIXME: temporary hack */
        intf_PlaylistPrev( p_main->p_playlist );
        intf_PlaylistPrev( p_main->p_playlist );
        p_input_bank->pp_input[0]->b_eof = 1;
    }
}

void KInterface::slotNext()
{
    if( p_input_bank->pp_input[0] != NULL )
    {
        /* FIXME: temporary hack */
        p_input_bank->pp_input[0]->b_eof = 1;
    }
}

void KInterface::slotSlow()
{
    if( p_input_bank->pp_input[0] != NULL )
    {
        input_SetStatus( p_input_bank->pp_input[0], INPUT_STATUS_SLOWER );

        vlc_mutex_lock( &p_main->p_playlist->change_lock );
        p_main->p_playlist->b_stopped = 0;
        vlc_mutex_unlock( &p_main->p_playlist->change_lock );
    }
}

void KInterface::slotEjectDisc()
{
    char *psz_device = NULL;
    char *psz_parser;

    /*
     * Get the active input
     * Determine whether we can eject a media, ie it's a VCD or DVD
     * If it's neither a VCD nor a DVD, then return
     */

    /*
     * Don't really know if I must lock the stuff here, we're using it read-only
     */

    if (p_main->p_playlist->current.psz_name != NULL)
    {
        if( !strncmp(p_main->p_playlist->current.psz_name, "dvd:", 4) )
        {
            switch( p_main->p_playlist->current.psz_name[4] )
            {
            case '\0':
            case '@':
                psz_device = config_GetPszVariable( "dvd" );
                break;
            default:
                /* Omit the first 4 characters */
                psz_device = strdup( p_main->p_playlist->current.psz_name + 4 );
                break;
            }
        }
        else if( !strncmp(p_main->p_playlist->current.psz_name, "vcd:", 4) )
        {
            switch( p_main->p_playlist->current.psz_name[4] )
            {
            case '\0':
            case '@':
                psz_device = config_GetPszVariable( "vcd" );
                break;
            default:
                /* Omit the first 4 characters */
                psz_device = strdup( p_main->p_playlist->current.psz_name + 4 );
                break;
            }
        }
        else
        {
            psz_device = strdup( p_main->p_playlist->current.psz_name );
        }
    }

    if( psz_device == NULL )
    {
        return;
    }

    /* Remove what we have after @ */
    psz_parser = psz_device;
    for( psz_parser = psz_device ; *psz_parser ; psz_parser++ )
    {
        if( *psz_parser == '@' )
        {
            *psz_parser = '\0';
            break;
        }
    }

    /* If there's a stream playing, we aren't allowed to eject ! */
    if( p_input_bank->pp_input[0] == NULL )
    {
        intf_WarnMsg( 4, "intf: ejecting %s", psz_device );

        intf_Eject( psz_device );
    }

    free(psz_device);
    return;

}

void KInterface::slotFullscreen()
{
    if( p_vout_bank->i_count )
    {
        vlc_mutex_lock( &p_vout_bank->pp_vout[0]->change_lock );

        p_vout_bank->pp_vout[0]->i_changes |= VOUT_FULLSCREEN_CHANGE;

        vlc_mutex_unlock( &p_vout_bank->pp_vout[0]->change_lock );

    }
}

void KInterface::slotFast()
{
    if( p_input_bank->pp_input[0] != NULL )
    {
        input_SetStatus( p_input_bank->pp_input[0], INPUT_STATUS_FASTER );

        vlc_mutex_lock( &p_main->p_playlist->change_lock );
        p_main->p_playlist->b_stopped = 0;
        vlc_mutex_unlock( &p_main->p_playlist->change_lock );
    }
}


void KInterface::slotChapterNext()
{
    input_area_t *  p_area;

    p_area = p_input_bank->pp_input[0]->stream.p_selected_area;

    if( p_area->i_part < p_area->i_part_nb )
    {
        p_area->i_part++;
        input_ChangeArea( p_input_bank->pp_input[0], (input_area_t*)p_area );

        input_SetStatus( p_input_bank->pp_input[0], INPUT_STATUS_PLAY );

        p_intf->p_sys->b_chapter_update = 1;
        vlc_mutex_lock( &p_input_bank->pp_input[0]->stream.stream_lock );
        slotUpdateMenus();
        vlc_mutex_unlock( &p_input_bank->pp_input[0]->stream.stream_lock );
    }
}

void KInterface::slotChapterPrev()
{
    input_area_t *  p_area;

    p_area = p_input_bank->pp_input[0]->stream.p_selected_area;

    if( p_area->i_part > 0 )
    {
        p_area->i_part--;
        input_ChangeArea( p_input_bank->pp_input[0], (input_area_t*)p_area );

        input_SetStatus( p_input_bank->pp_input[0], INPUT_STATUS_PLAY );

        p_intf->p_sys->b_chapter_update = 1;
        vlc_mutex_lock( &p_input_bank->pp_input[0]->stream.stream_lock );
        slotUpdateMenus();
        vlc_mutex_unlock( &p_input_bank->pp_input[0]->stream.stream_lock );
    }
}

void KInterface::slotTitleNext()
{
    input_area_t *  p_area;
    int             i_id;

    i_id = p_input_bank->pp_input[0]->stream.p_selected_area->i_id + 1;

    if( i_id < p_input_bank->pp_input[0]->stream.i_area_nb )
    {
        p_area = p_input_bank->pp_input[0]->stream.pp_areas[i_id];   
        input_ChangeArea( p_input_bank->pp_input[0], (input_area_t*)p_area );

        input_SetStatus( p_input_bank->pp_input[0], INPUT_STATUS_PLAY );

        p_intf->p_sys->b_title_update = 1;
        vlc_mutex_lock( &p_input_bank->pp_input[0]->stream.stream_lock );
        slotUpdateMenus();
        vlc_mutex_unlock( &p_input_bank->pp_input[0]->stream.stream_lock );
    }

}

void KInterface::slotTitlePrev()
{
    input_area_t *  p_area;
    int             i_id;

    i_id = p_input_bank->pp_input[0]->stream.p_selected_area->i_id - 1;

    /* Disallow area 0 since it is used for video_ts.vob */
    if( i_id > 0 )
    {
        p_area = p_input_bank->pp_input[0]->stream.pp_areas[i_id];
        input_ChangeArea( p_input_bank->pp_input[0], (input_area_t*)p_area );

        input_SetStatus( p_input_bank->pp_input[0], INPUT_STATUS_PLAY );

        p_intf->p_sys->b_title_update = 1;
        vlc_mutex_lock( &p_input_bank->pp_input[0]->stream.stream_lock );
        slotUpdateMenus();
        vlc_mutex_unlock( &p_input_bank->pp_input[0]->stream.stream_lock );
    }
}

/* Stuff */


/*****************************************************************************
 * LanguageMenus: update interactive menus of the interface
 *****************************************************************************
 * Sets up menus with information from input:
 *  - languages
 *  - sub-pictures
 * Warning: since this function is designed to be called by management
 * function, the interface lock has to be taken
 *****************************************************************************/
void KInterface::LanguageMenu( KSelectAction * Root,
                          es_descriptor_t * p_es, int i_cat )
{
//    TMenuItem     * Separator;
//    TMenuItem     * Item;
    int      ItemActive;
    QStringList *items;
    QString      Name;
    int             i_item;
    int             i;

    items = new QStringList();

    /* remove previous menu */
    Root->clear();
    Root->setEnabled( false );

    /* special case for "off" item */
    Name = "None";
    items->append( Name );
    
    /* separator item */
    //Separator = new TMenuItem( Root );
    //Separator->Caption = "-";
    //Root->Add( Separator );

    ItemActive = -1;
    i_item = 0;


#define ES p_input_bank->pp_input[0]->stream.pp_es[i]
    /* create a set of language buttons and append them to the Root */
    for( i = 0; i < p_input_bank->pp_input[0]->stream.i_es_number; i++ )
    {
        if( ( ES->i_cat == i_cat ) &&
            ( !ES->p_pgrm ||
              ES->p_pgrm ==
                 p_input_bank->pp_input[0]->stream.p_selected_program ) )
        {
            i_item++;
            Name = p_input_bank->pp_input[0]->stream.pp_es[i]->psz_desc;
            if( Name.isEmpty() )
            {
                Name.sprintf( "Language %d", i_item );
            }

            if( p_es == p_input_bank->pp_input[0]->stream.pp_es[i] )
            {
                /* don't lose Item when we append into menu */
                ItemActive = i_item;
            }

            /* setup signal hanling */
            items->append( Name );
        }
    }
#undef ES


    Root->setItems( *items );
    /* check currently selected item */
    if( ItemActive != -1 )
    {
        Root->setCurrentItem(ItemActive);
    }

    /* be sure that menu is enabled if non empty */
    if( i_item > 0 )
    {
        Root->setEnabled( true );
    }

}


/*****************************************************************************
 * ModeManage: actualise the aspect of the interface whenever the input
 *             changes.
 *****************************************************************************
 * The lock has to be taken before you call the function.
 *****************************************************************************/
void KInterface::ModeManage()
{
    bool            b_control;

    /* hide all boxes */
    //    GroupBoxFile->Visible = false;
    //    GroupBoxNetwork->Visible = false;
    dvdToolBar->hide();
//    toolBar( "dvdToolBar" )->setEnabled( false );

    /* hide slider */
    fSlider->hide();
            
    /* controls unavailable */
    b_control = 0;

    /* show the box related to current input mode */
    if( p_input_bank->pp_input[0] != NULL )
    {
        switch( p_input_bank->pp_input[0]->stream.i_method & 0xf0 )
        {    
            case INPUT_METHOD_FILE:
                //                GroupBoxFile->Visible = true;
                //ActiveGB = GroupBoxFile;
                //LabelFileName->Caption =
                //p_input_bank->pp_input[0]->psz_source;
                break;
            case INPUT_METHOD_DISC:
                intf_WarnMsg(1, "showing dvd toolbar");

                dvdToolBar->show();
                dvdToolBar->setEnabled( true );
                //                GroupBoxDisc->Visible = true;
                //                ActiveGB = GroupBoxDisc;
                break;
            case INPUT_METHOD_NETWORK:
                //GroupBoxNetwork->Visible = true;
                //ActiveGB = GroupBoxNetwork;
                //LabelServer->Caption = p_input_bank->pp_input[0]->psz_source;
                //if( config_GetIntVariable( "network-channel" ) )
                //{
                //    LabelChannel->Visible = true;
                //}
                //else
                //{
                //    LabelChannel->Visible = false;
                //}
                break;
            default:
                intf_WarnMsg( 3, "intf: can't determine input method" );
                //GroupBoxFile->Visible = true;
                //ActiveGB = GroupBoxFile;
                //LabelFileName->Caption = p_input_bank->pp_input[0]->psz_source;
                break;
        }

        //i_Height = StatusBar->Height + ActiveGB->Height + ToolBar->Height + 47;

        /* initialize and show slider for seekable streams */
        if( p_input_bank->pp_input[0]->stream.b_seekable )
        {
            intf_WarnMsg(1, "before the stupid showing slider");

            fSlider->show();
            intf_WarnMsg(1, "finnished showing slider");
        }

        /* control buttons for free pace streams */
        b_control = p_input_bank->pp_input[0]->stream.b_pace_control;
        
        /* get ready for menu regeneration */
        p_intf->p_sys->b_program_update = 1;
        p_intf->p_sys->b_title_update = 1;
        p_intf->p_sys->b_chapter_update = 1;
        p_intf->p_sys->b_audio_update = 1;
        p_intf->p_sys->b_spu_update = 1;

        intf_WarnMsg( 3, "intf: stream has changed, refreshing interface" );
    }
    else
    {
        //if( config_GetIntVariable( "network-channel" ) )
        //{
        //GroupBoxNetwork->Visible = true;
        //    LabelChannel->Visible = true;
        //    i_Height += GroupBoxNetwork->Height;
        //}
        //else
        //{
        /* unsensitize menus */
//        MenuProgram->Enabled = false;
//        MenuTitle->Enabled = false;
//        MenuChapter->Enabled = false;
        audio->setEnabled( false );
        subtitles->setEnabled( false );
//        PopupNavigation->Enabled = false;
//        PopupAudio->Enabled = false;
//        PopupSubtitles->Enabled = false;
        //}
    }

    /* resize main window */
    //  this->Height = i_Height;

    /* set control items */
    backward->setEnabled( false );
    stop->setEnabled( true );
    ejectDisc->setEnabled( !b_control );
    pause->setEnabled( b_control );
    slow->setEnabled( b_control );
    fast->setEnabled( b_control );

}
